"""
Module for the plpipes package.

This module serves as a container for the plpipes library. All the
interesting functionalities and features are implemented in the 
submodules of this package.
"""

from plpipes.config import cfg
import logging
